//
// Created by philipp on 04.04.18.
//

#ifndef PLATOONPROTOCOL_PROTOCOLFACADE_H
#define PLATOONPROTOCOL_PROTOCOLFACADE_H

#include <NetworkingLib/Networking.h>
#include "Vehicle.h"

namespace platoonProtocol
{

class VehicleFacade
{
public:
    explicit VehicleFacade(Vehicle::Role role, networking::Networking & net, const NetworkInfo & info);

    void reset(Vehicle::Role role);

    Vehicle::Ptr get();

    // Vehicle methods

    bool isIdle();

    bool isPlatoonRunning();

    void createPlatoon();

    void leavePlatoon();

    void stop();

    VehicleId getOwnVehicleId();

    Vehicle::Role getRole();
    
    void setCallback(const Vehicle::Callback & callback);

    // LeaderVehicle methods

    std::size_t getNumFollowers();

    void setPlatoonConfig(const PlatoonConfig & platoonConfig);

    void setPlatoonSpeed(PlatoonSpeed platoonSpeed);

    void setInnerPlatoonDistance(InnerPlatoonDistance innerPlatoonDistance);

    // FollowerVehicle methods

    networking::time::TimedValue<PlatoonConfig> getPlatoonConfig();
    
private:
    Vehicle::Ptr vehicle;
    std::mutex mutex;
    
    networking::Networking & net;
    NetworkInfo info;
    Vehicle::Callback callback{Vehicle::DEFAULT_CALLBACK};
        
};

}

#endif //PLATOONPROTOCOL_PROTOCOLFACADE_H
